@@ -28,7 +28,9 @@ REQUEST_ID = 'req-test-request-id'
 def _stub_volume(*args, **kwargs):
     volume = {
         "migration_status": None,
-        "attachments": [{u'server_id': u'1234'}],
+        "attachments": [{u'server_id': u'1234',
+                         u'id': u'3f88836f-adde-4296-9f6b-2c59a0bcda9a',
+                         u'attachment_id': u'5678'}],
         "links": [
             {
                 "href": "http://localhost/v2/fake/volumes/1234",
@@ -369,7 +371,7 @@ class FakeHTTPClient(base_client.HTTPClient):
         # add fake request-id header
         headers['x-openstack-request-id'] = REQUEST_ID
         if self.version_header:
-            headers['OpenStack-API-version'] = version_header
+            headers['OpenStack-API-version'] = self.version_header
         r = utils.TestResponse({
             "status_code": status,
             "text": body,
@@ -387,9 +389,12 @@ class FakeHTTPClient(base_client.HTTPClient):
     #
 
     def get_snapshots_detail(self, **kw):
+        if kw.get('with_count', False):
+            return (200, {}, {'snapshots': [
+                _stub_snapshot(),
+            ], 'count': 1})
         return (200, {}, {'snapshots': [
-            _stub_snapshot(),
-        ]})
+            _stub_snapshot()]})
 
     def get_snapshots_1234(self, **kw):
         return (200, {}, {'snapshot': _stub_snapshot(id='1234')})
@@ -462,6 +467,10 @@ class FakeHTTPClient(base_client.HTTPClient):
             ]})
 
     def get_volumes_detail(self, **kw):
+        if kw.get('with_count', False):
+            return (200, {}, {"volumes": [
+                _stub_volume(id=kw.get('id', 1234))
+            ], "count": 1})
         return (200, {}, {"volumes": [
             _stub_volume(id=kw.get('id', 1234))
         ]})
@@ -535,10 +544,21 @@ class FakeHTTPClient(base_client.HTTPClient):
         elif action == 'os-volume_upload_image':
             assert 'image_name' in body[action]
             _body = body
+        elif action == 'revert':
+            assert 'snapshot_id' in body[action]
         else:
             raise AssertionError("Unexpected action: %s" % action)
         return (resp, {}, _body)
 
+    def get_volumes_fake(self, **kw):
+        r = {'volume': self.get_volumes_detail(id='fake')[2]['volumes'][0]}
+        return (200, {}, r)
+
+    def post_volumes_fake_action(self, body, **kw):
+        _body = None
+        resp = 202
+        return (resp, {}, _body)
+
     def post_volumes_5678_action(self, body, **kw):
         return self.post_volumes_1234_action(body, **kw)
 
@@ -625,7 +645,6 @@ class FakeHTTPClient(base_client.HTTPClient):
                           'gigabytes': 1,
                           'backups': 1,
                           'backup_gigabytes': 1,
-                          'consistencygroups': 1,
                           'per_volume_gigabytes': 1, }})
 
     def get_os_quota_sets_test_defaults(self):
@@ -637,7 +656,6 @@ class FakeHTTPClient(base_client.HTTPClient):
                           'gigabytes': 1,
                           'backups': 1,
                           'backup_gigabytes': 1,
-                          'consistencygroups': 1,
                           'per_volume_gigabytes': 1, }})
 
     def put_os_quota_sets_test(self, body, **kw):
@@ -652,7 +670,6 @@ class FakeHTTPClient(base_client.HTTPClient):
                           'gigabytes': 1,
                           'backups': 1,
                           'backup_gigabytes': 1,
-                          'consistencygroups': 2,
                           'per_volume_gigabytes': 1, }})
 
     def delete_os_quota_sets_1234(self, **kw):
@@ -668,28 +685,22 @@ class FakeHTTPClient(base_client.HTTPClient):
     def get_os_quota_class_sets_test(self, **kw):
         return (200, {}, {'quota_class_set': {
                           'class_name': 'test',
-                          'metadata_items': [],
                           'volumes': 1,
                           'snapshots': 1,
                           'gigabytes': 1,
                           'backups': 1,
                           'backup_gigabytes': 1,
-                          'consistencygroups': 1,
                           'per_volume_gigabytes': 1, }})
 
     def put_os_quota_class_sets_test(self, body, **kw):
         assert list(body) == ['quota_class_set']
-        fakes.assert_has_keys(body['quota_class_set'],
-                              required=['class_name'])
+        fakes.assert_has_keys(body['quota_class_set'])
         return (200, {}, {'quota_class_set': {
-                          'class_name': 'test',
-                          'metadata_items': [],
                           'volumes': 2,
                           'snapshots': 2,
                           'gigabytes': 1,
                           'backups': 1,
                           'backup_gigabytes': 1,
-                          'consistencygroups': 2,
                           'per_volume_gigabytes': 1}})
 
     #
@@ -881,6 +892,12 @@ class FakeHTTPClient(base_client.HTTPClient):
         tenant_id = '0fa851f6668144cf9cd8c8419c1646c1'
         backup1 = '76a17945-3c6f-435c-975b-b5685db10b62'
         backup2 = 'd09534c6-08b8-4441-9e87-8976f3a8f699'
+        if kw.get('with_count', False):
+            return (200, {},
+                    {'backups': [
+                        _stub_backup_full(backup1, base_uri, tenant_id),
+                        _stub_backup_full(backup2, base_uri, tenant_id)],
+                    'count': 2})
         return (200, {},
                 {'backups': [
                     _stub_backup_full(backup1, base_uri, tenant_id),
@@ -1079,9 +1096,9 @@ class FakeHTTPClient(base_client.HTTPClient):
             },
         ]
         if host:
-            services = filter(lambda i: i['host'] == host, services)
+            services = [i for i in services if i['host'] == host]
         if binary:
-            services = filter(lambda i: i['binary'] == binary, services)
+            services = [i for i in services if i['binary'] == binary]
         return (200, {}, {'services': services})
 
     def put_os_services_enable(self, body, **kw):
